----------------------------------------------------------
--SuperSlider Manipulator
--Version 0.1
--Started: 12/22/00
--Edited:  12/24/00
--by Borislav Petrov [Boboland]
--http://gfxcentral.com/bobo/
--Original Slider script by Scott Morrison, July 25, 2000
--Based on Peter Watje's 2d slider object
----------------------------------------------------------

plugin simpleManipulator superSliderManipulator
    name:"SuperSlider"
    classID:#(0x589e68c1, 0x7eba7edb)
    category:"Manipulators"
(
    -- sliderOffset is the distance in pixels from the "+" to the slider bar
    local sliderOffset = 9

    -- textYOffset is the vertical distance from the slider to the text
    local textYOffset = 10

    -- textXOffset is the horizantal distance from the slider to the text
    local textXOffset = -5

    -- The minimum width a slider can have in pixels
    local minWidth = 10


    parameters main rollout:paramRollout
    ( 
         -- The value of the slider
         value      type:#float   animatable:true  ui:spnValue  default: 0.0

         -- The min and max values the slider can take
         minVal     type:#float   animatable:true  ui:spnMin    default: 0.0
         maxVal     type:#float   animatable:true  ui:spnMax    default: 100.0

         -- The slider position as a percentage of screen space
         xPos       type:#float   animatable:false ui:spnXPos   default: 0.0
         yPos       type:#float   animatable:false ui:spnYPos   default: 0.0

         -- The width of the slider in pixels
         width      type:#float   animatable:false ui:spnWidth  default: 100.0

         -- When the hide flag is true, the slider bar and value are not draw
         hide       type:#boolean animatable:false ui:checkHide default: false

         -- The display name of the slider
         sldName    type:#string  animatable:false              default: ""

        -- Snap toggle
         snapToggle type:#boolean animatable:false ui:checkSnap default: true
		 
        -- Ticks toggle
         ticksToggle type:#boolean animatable:false ui:checkTicks default: false

         -- snap value
         snapVal    type:#float   animatable:false  ui:spnSnapVal default: 1.0
		 
         -- ticks value
         TicksVal    type:#integer animatable:false  ui:spnTicksVal default:2
		 
		 -- User Snaps
         UserSnapToggle type:#boolean animatable:false ui:checkUserSnap default: true
 		 UserSnapSens type:#float animatable:false  ui:spnUserSnapSens default:10.0

         UserSnapOrigin  type:#float animatable:false  ui:spnOrigin default:0.0
         UserSnapVal_1  type:#float animatable:false  ui:spnUSnap01 default:50.0
         UserSnapVal_2  type:#float animatable:false  ui:spnUSnap02 default:100.0
         UserSnapVal_3  type:#float animatable:false  ui:spnUSnap03 default:-50.0
         UserSnapVal_4  type:#float animatable:false  ui:spnUSnap04 default:-100.0
		 
		 
		 -- user colors
		 UserGreenColor		type:#color animatable:false	ui:UserGreenColor	default:(color 0 255 0)
		 UserYellowColor	type:#color animatable:false	ui:UserYellowColor	default:(color 255 255 0)
		 UserRedColor		type:#color animatable:false	ui:UserRedColor	default:(color 255 0 0)
		 UseCustomColors	type:#boolean animatable:false ui:UseCustomColors default: false

    )

    -- The rollout for the slider parameters
    rollout paramRollout "Parameters"
    (
		Group "Slider Controls"
		(
		Label sliderName "Name:" across:2 align:#left
        Checkbox checkHide  "Hide"        checked:false align:#right
        Edittext nameEdit   ""        text:""	height:35 offset:[-5,0]
        Spinner  spnValue   "Value:"       range:[-1.0e30, 1.0e30, 0.0] 
        Spinner  spnMin     "Minimum:"     range:[-1.0e30, 1.0e30, 0.0] 
        Spinner  spnMax     "Maximum:"     range:[-1.0e30, 1.0e30, 100.0] 
		)
		
		Group "Slider Size And Placement"
		(
        Spinner  spnXPos    "X Position:"  range:[0.0, 1.0, 0.0] scale:0.01
        Spinner  spnYPos    "Y Position:"  range:[0.0, 1.0, 0.0] scale:0.01
        Spinner  spnWidth   "Width:"       range:[10.0, 1000.0, 100.0] 
		)		
		
		Group "Snap and Ticks"
		(
        Checkbutton checkSnap  "Use Snaps"  across:2 align:#left width:68
        Checkbutton checkTicks  "Show Ticks"  align:#right width:68 

        Spinner  spnSnapVal "Snap Value:"  range:[0.0, 10000.0, 1.0] 
        Spinner  spnTicksVal "Major Ticks"  range:[2, 100, 10] type:#integer
		)

		fn filter_sliders obj = ( FindItem (getPropNames obj) #UserGreenColor > 0 )	

		Group "User Snaps"
		(
        Checkbox checkUserSnap  "Use" across:2 align:#left offset:[0,-2]
        Spinner  spnUserSnapSens "Strength:"  range:[0.0, 1000.0, 10.0] fieldwidth:40 align:#right type:#integer
        Spinner  spnOrigin "Origin:"  range:[-10000.0, 10000.0, 0.0] 
        Spinner  spnUSnap01 "UserSnap 1:"  range:[-10000.0, 10000.0, 50.0] 
        Spinner  spnUSnap02 "UserSnap 2:"  range:[-10000.0, 10000.0, 100.0] 
        Spinner  spnUSnap03 "UserSnap 3:"  range:[-10000.0, 10000.0, -50.0] 
        Spinner  spnUSnap04 "UserSnap 4:"  range:[-10000.0, 10000.0, -100.0] 
		PickButton getUSnaps "Acquire USnaps From..." width:140 filter:filter_sliders
		)
		

		
		
		Group "Custom Colors"
		(
		checkbox UseCustomColors "Use" align:#left across:2
		ColorPicker UserGreenColor	"Active"		align:#right fieldwidth:40
		ColorPicker UserYellowColor	"Inactive"		align:#right fieldwidth:40
		ColorPicker UserRedColor	"Selected"		align:#right fieldwidth:40
		PickButton getColors "Acquire Colors From..." width:140 filter:filter_sliders
		)

		On getColors picked obj do
		(
		UserGreenColor.color = obj.UserGreenColor
		UserYellowColor.color = obj.UserYellowColor
		UserRedColor.color = obj.UserRedColor
		UseCustomColors.checked = obj.UseCustomColors
		)
		
		
		
		On getUSnaps picked obj do
		(
		spnOrigin.value = obj.UserSnapOrigin  
		spnUSnap01.value = obj.UserSnapVal_1
		spnUSnap02.value = obj.UserSnapVal_2
		spnUSnap03.value = obj.UserSnapVal_3
		spnUSnap04.value = obj.UserSnapVal_4
		checkUserSnap.checked = obj.UserSnapToggle 
		)		
		
        on nameEdit entered txt do 
        (
           sldName = txt
        )

        on paramRollout open do 
        (
           nameEdit.text = sldName
        )
    )

    -- This manipualtor manipulates itself
    on canManipulate target  return (classOf target == superSliderManipulator)

    -- Creation mouse procedure
    tool create
    (
        on mousePoint click do
           case click of
           (
              1: 
              (
                -- Compute the percentage of of the mouse point in screen space
                this.xPos = viewPoint.x / gw.getWinSizeX()
                this.yPos = viewPoint.y / gw.getWinSizeY()

                #stop 
              )
           )

    )

    -- Create the manipulator gizmos.
    on updateGizmos do
    (
	
	
        -- Clear the current gizmo cache
        this.clearGizmos()

        -- If this is not a stand-alone manip, get values from the manip target
       if (target != undefined) then 
        (
           this.value      = target.value
           this.minVal     = target.minVal
           this.maxVal     = target.maxVal
           this.xPos       = target.xPos
           this.yPos       = target.yPos
           this.width      = target.width
           this.hide       = target.hide
           this.sldName    = target.sldName
           this.snapToggle = target.snapToggle
		   this.ticksToggle= target.ticksToggle
           this.snapVal    = target.snapVal
           this.ticksVal    = target.ticksVal

         this.UserSnapToggle = target.UserSnapToggle 
		 this.UserSnapSens = target.UserSnapSens
         this.UserSnapOrigin  =target.UserSnapOrigin  
         this.UserSnapVal_1  = target.UserSnapVal_1  
         this.UserSnapVal_2  = target.UserSnapVal_2  
         this.UserSnapVal_3 = target.UserSnapVal_3 
         this.UserSnapVal_4  = target.UserSnapVal_4  

		   
		   if target.UseCustomColors then
		   (
		   UGreenColor   =  target.UserGreenColor / 255.0
		   URedColor =    target.UserRedColor / 255.0
		   )
		   else
		   (
    		UGreenColor   =  colorMan.getColor #manipulatorsActive   
		    URedColor =    colorMan.getColor #manipulatorsSelected
		   )
        ) 
        else
        (
		   if this.UseCustomColors then
			(
			UGreenColor   =  this.UserYellowColor / 255.0
		    URedColor =    this.UserRedColor / 255.0
			)
			else
			(
    		UGreenColor   =  colorMan.getColor #manipulatorsInactive
		    URedColor =    colorMan.getColor #manipulatorsSelected
			)
        )
		
        -- Compute the origin of the slider in screen space
        local x = xPos * gw.getWinSizeX()
        local y = yPos * gw.getWinSizeY()

        -- These flags are used for all the gizmos
        local flags = (gizmoUseScreenSpace + gizmoActiveViewportOnly)

        -- Gizmo 0
        -- This is the box that you use the move the slider
        local pos = [x, y, 0.0]
        this.addGizmoMarker #hollowBox pos flags UGreenColor URedColor

        -- Gizmo 1
        -- This is the plus-sign used to toggle the hide flag
        pos =  [x + sliderOffset, y, 0.0]
        this.addGizmoMarker #plusSign pos flags UGreenColor URedColor

        -- Compute the text label
        local str

        if (hide) then str = sldName
        else 
        (
            if sldName.count > 0 then str = sldName + ": " + (value as string)
            else str = value as string
        )

        local textLoc = [x + textXOffset, y - textYOffset, 0.0f]

        -- Gizmo 2
        -- The text with the name and value
        this.addGizmoText str textLoc flags UGreenColor URedColor

        -- If the slider is hidden, don't draw any more
        if hide then return ""

        -- Draw the slider bar
        local barStart = x + 2 * sliderOffset
        local barPos = [barStart, y, 0.0]
        local barEnd = [barStart + this.width, y, 0.0]
 
        -- Create a polyline gizmo
        local giz = manip.makeGizmoShape()
        giz.AddPoint barPos
        giz.AddPoint barEnd
        this.addGizmoShape giz flags UGreenColor  UGreenColor   
		

        -- Draw the slider "thumb" gizmo
        local relVal = (value - minVal) / (maxVal - minVal)
        local valPos = [barStart + relVal * width, y + 10, 0.0f]

        -- Gizmo 4
        -- The slider thumb triangle
        this.addGizmoMarker #triangle valPos flags UGreenColor URedColor

        local resizePos = [barStart + width + sliderOffset, y, 0.0f]
        
        -- Gizmo 5
        -- The slider resize diamond
        this.addGizmoMarker #diamond resizePos flags UGreenColor URedColor
		
	
		--If UserSnaps are enabled, draw...
		if UserSnapToggle then
		(
		
		--Gizmo 6
        local giz = manip.makeGizmoShape()	
		if maxVal >= UserSnapOrigin and minVal <= UserSnapOrigin then
		(
			relVal = (UserSnapOrigin - minVal) / (maxVal - minVal)
	        giz.AddPoint [barStart +relVal*width,y-10,0]
	        giz.AddPoint [barStart +relVal*width,y+8,0]
		)
		else
		(
	        giz.AddPoint [barStart,y,0]
		)
        this.addGizmoShape giz flags UGreenColor  URedColor

		--Gizmo 7

        local giz = manip.makeGizmoShape()		
		if maxVal >= UserSnapVal_1 and minVal <= UserSnapVal_1 then
		(
			relVal = (UserSnapVal_1- minVal) / (maxVal - minVal)
	        giz.AddPoint [barStart +relVal*width,y-8,0]
	        giz.AddPoint [barStart +relVal*width,y+8,0]
		)
		else
		(
	        giz.AddPoint [barStart,y,0]
		)
        this.addGizmoShape giz flags UGreenColor  URedColor

		--Gizmo 8
		
        local giz = manip.makeGizmoShape()		
		if maxVal >= UserSnapVal_2 and minVal <= UserSnapVal_2 then
		(
			relVal = (UserSnapVal_2- minVal) / (maxVal - minVal)
	        giz.AddPoint [barStart +relVal*width,y-7,0]
	        giz.AddPoint [barStart +relVal*width,y+8,0]
		)
		else
		(
	        giz.AddPoint [barStart,y,0]
		)	
        this.addGizmoShape giz flags UGreenColor  URedColor

		--Gizmo 9

        local giz = manip.makeGizmoShape()		
		if maxVal >= UserSnapVal_3 and minVal <= UserSnapVal_3 then
		(
			relVal = (UserSnapVal_3- minVal) / (maxVal - minVal)
	        giz.AddPoint [barStart +relVal*width,y-6,0]
	        giz.AddPoint [barStart +relVal*width,y+8,0]
		)
		else
		(
	        giz.AddPoint [barStart,y,0]
		)	
        this.addGizmoShape giz flags UGreenColor  URedColor

		--Gizmo 10
		
        local giz = manip.makeGizmoShape()		
		if maxVal >= UserSnapVal_3 and minVal <= UserSnapVal_3 then
		(
			relVal = (UserSnapVal_4- minVal) / (maxVal - minVal)
	        giz.AddPoint [barStart +relVal*width,y-5,0]
	        giz.AddPoint [barStart +relVal*width,y+8,0]
		)	
		else
		(
	        giz.AddPoint [barStart,y,0]
		)	
        this.addGizmoShape giz flags UGreenColor  URedColor
	)	


		
		if TicksToggle and (maxVal-minVal)/SnapVal <= 100  then
		(
		majorLineCounter = 0
		majorLine = TicksVal
		for i = minVal to maxVal by SnapVal do
		(
        local giz = manip.makeGizmoShape()
		relVal = (i - minVal) / (maxVal - minVal)
		majorLineCounter += 1
		if majorLineCounter == majorLine then majorLineCounter = 0
		if majorLineCounter == 1 then
		(
        giz.AddPoint [barStart +relVal*width,y-0,0]
        giz.AddPoint [barStart +relVal*width,y+8,0]
		)
		else
		(
        giz.AddPoint [barStart +relVal*width,y-0,0]
        giz.AddPoint [barStart +relVal*width,y+5,0]
		)
        this.addGizmoShape giz flags UGreenColor  UGreenColor   
		)
		)
	
        -- return an empty string so no tool tip is displayed
        return ""
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the slider
    on mouseMove m which do
    (
        local xSize = gw.getWinSizeX()
        local ySize = gw.getWinSizeY()

        if (which == 0)  then 
        (
            -- Gizmo 0

            -- Set the slider postition
            target.xPos = m.x / xSize
            target.yPos = m.y / ySize
        ) 
        else if (which == 4) then 
        (
            -- Gizmo 4

            -- Set the slider value
            local x = xPos * xSize
            local barStart = x + 2.0 * sliderOffset
            local newVal = (m.x - barStart) / width
            newVal = minVal + newVal * (maxVal - minVal)

            -- Clamp the value
            if (newVal < minVal) then newVal = minVal
            if (newVal > maxVal) then newVal = maxVal

            -- Snap, if needed
            if (snapToggle) then newVal = newVal - (mod newVal snapVal)

            if UserSnapToggle and abs(newVal-UserSnapOrigin) < UserSnapSens then newVal = UserSnapOrigin
            if UserSnapToggle and abs(newVal-UserSnapVal_1) < UserSnapSens then newVal = UserSnapVal_1
            if UserSnapToggle and abs(newVal-UserSnapVal_2) < UserSnapSens then newVal = UserSnapVal_2
            if UserSnapToggle and abs(newVal-UserSnapVal_3) < UserSnapSens then newVal = UserSnapVal_3
            if UserSnapToggle and abs(newVal-UserSnapVal_4) < UserSnapSens then newVal = UserSnapVal_4

            -- Set the value in the target
            target.value = newVal

        ) 
        else if (which == 5) then 
        (
            -- Gizmo 4
            -- Set the slider width
            local x = xPos * xSize
            local barStart = x + 2.0 * sliderOffset
            local newWidth = m.x - sliderOffset - barStart

            -- Clamp the width
            if (newWidth < minWidth) then newWidth = 10.0

            -- Set the width in the target
            target.width = newWidth
        )
    )

    on mouseDown m which do
    (
      -- Toggle the value of the "Hide" state
      if (which == 1) then target.hide = not hide 

	  --Snap to UserSnaps
      if (which == 6) then target.value = UserSnapOrigin
      if (which == 7) then target.value = UserSnapVal_1
      if (which == 8) then target.value  = UserSnapVal_2
      if (which == 9) then target.value  = UserSnapVal_3
      if (which == 10) then target.value  = UserSnapVal_4
    )
)
